==> flutter_widget_from_html-0.4.0-rc.2020021401.tar.gz: lib/src/widget_factory.dart <==
import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:flutter_widget_from_html_core/flutter_widget_from_html_core.dart'
    as core;
import 'package:url_launcher/url_launcher.dart';

import 'data_classes.dart';
import 'html_widget.dart';
import 'video_player.dart';
import 'web_view.dart';

part 'ops/tag_a_extended.dart';
part 'ops/tag_iframe.dart';
part 'ops/tag_svg.dart';
part 'ops/tag_video.dart';

class WidgetFactory extends core.WidgetFactory {
  final HtmlWidgetConfig _config;

  BuildOp _tagAExtended;
  BuildOp _tagIframe;
  BuildOp _tagSvg;
  BuildOp _tagVideo;

  WidgetFactory(this._config) : super(_config);

  @override
  Widget buildDivider() => const Divider(height: 1);

  @override
  Iterable<Widget> buildGestureDetectors(
    BuilderContext bc,
    Iterable<Widget> widgets,
    GestureTapCallback onTap,
  ) =>
      widgets.map((widget) => InkWell(child: widget, onTap: onTap));

  @override
  GestureTapCallback buildGestureTapCallbackForUrl(String url) => url != null
      ? () => _config.onTapUrl != null
          ? _config.onTapUrl(url)
          : canLaunch(url).then((ok) => ok ? launch(url) : null)
      : null;

  @override
  ImageProvider buildImageFromUrl(String url) =>
      url?.isNotEmpty == true ? CachedNetworkImageProvider(url) : null;

  Widget buildVideoPlayer(
    String url, {
    bool autoplay,
    bool controls,
    double height,
    bool loop,
    double width,
  }) {
    final dimensOk = height != null && height > 0 && width != null && width > 0;
    return VideoPlayer(
      url,
      aspectRatio: dimensOk ? width / height : 16 / 9,
      autoResize: !dimensOk,
      autoplay: autoplay,
      controls: controls,
      loop: loop,
    );
  }

  Widget buildWebView(
    String url, {
    double height,
    double width,
  }) {
    if (_config.webView != true) return buildWebViewLinkOnly(url);

    final dimensOk = height != null && height > 0 && width != null && width > 0;
    return WebView(
      url,
      aspectRatio: dimensOk ? width / height : 16 / 9,
      getDimensions: !dimensOk && _config.webViewJs == true,
      interceptNavigationRequest: (newUrl) {
        if (newUrl == url) return false;

        buildGestureTapCallbackForUrl(newUrl)();
        return true;
      },
      js: _config.webViewJs == true,
      unsupportedWorkaroundForIssue37:
          _config.unsupportedWebViewWorkaroundForIssue37 == true,
    );
  }

  Widget buildWebViewLinkOnly(String url) => GestureDetector(
        child: Text(url),
        onTap: buildGestureTapCallbackForUrl(url),
      );

  @override
  NodeMetadata parseLocalName(NodeMetadata meta, String localName) {
    switch (localName) {
      case 'a':
        meta = lazySet(meta, buildOp: tagAExtended());
        break;
      case 'iframe':
        // return asap to avoid being disabled by core
        return lazySet(meta, buildOp: tagIframe());
      case 'svg':
        // return asap to avoid being disabled by core
        return lazySet(meta, buildOp: tagSvg());
      case 'video':
        meta = lazySet(meta, buildOp: tagVideo());
        break;
    }

    return super.parseLocalName(meta, localName);
  }

  BuildOp tagAExtended() {
    _tagAExtended ??= _TagAExtended().buildOp;
    return _tagAExtended;
  }

  BuildOp tagIframe() {
    _tagIframe ??= _TagIframe(this).buildOp;
    return _tagIframe;
  }

  BuildOp tagSvg() {
    _tagSvg ??= _TagSvg(this).buildOp;
    return _tagSvg;
  }

  BuildOp tagVideo() {
    _tagVideo ??= _TagVideo(this).buildOp;
    return _tagVideo;
  }
}

==> flutter_widget_from_html-0.4.1-rc.2020052701.tar.gz: lib/src/widget_factory.dart <==
import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:flutter_widget_from_html_core/flutter_widget_from_html_core.dart'
    as core;
import 'package:url_launcher/url_launcher.dart';

import 'builder.dart';
import 'data.dart';
import 'helpers.dart';
import 'html_widget.dart';

part 'ops/tag_a_extended.dart';
part 'ops/tag_iframe.dart';
part 'ops/tag_svg.dart';
part 'ops/tag_video.dart';

/// A factory to build widget for HTML elements
/// with support for [WebView] and [VideoPlayer] etc.
class WidgetFactory extends core.WidgetFactory {
  BuildOp _tagAExtended;
  BuildOp _tagIframe;
  BuildOp _tagSvg;
  BuildOp _tagVideo;
  HtmlWidget _widget;

  @override
  Widget buildDivider() => const Divider(height: 1);

  @override
  Iterable<Widget> buildGestureDetectors(
    BuildContext _,
    Iterable<Widget> widgets,
    GestureTapCallback onTap,
  ) =>
      widgets.map((widget) => InkWell(child: widget, onTap: onTap));

  @override
  GestureTapCallback buildGestureTapCallbackForUrl(String url) {
    if (url == null) return null;
    if (_widget?.onTapUrl == null) {
      return () => canLaunch(url).then((ok) => ok ? launch(url) : null);
    }
    return () => _widget.onTapUrl(url);
  }

  @override
  ImageProvider buildImageFromUrl(String url) =>
      url?.isNotEmpty == true ? CachedNetworkImageProvider(url) : null;

  Widget buildVideoPlayer(
    String url, {
    bool autoplay,
    bool controls,
    double height,
    bool loop,
    double width,
  }) {
    final dimensOk = height != null && height > 0 && width != null && width > 0;
    return VideoPlayer(
      url,
      aspectRatio: dimensOk ? width / height : 16 / 9,
      autoResize: !dimensOk,
      autoplay: autoplay,
      controls: controls,
      loop: loop,
    );
  }

  Widget buildWebView(
    String url, {
    double height,
    double width,
  }) {
    if (_widget?.webView != true) return buildWebViewLinkOnly(url);

    final dimensOk = height != null && height > 0 && width != null && width > 0;
    return WebView(
      url,
      aspectRatio: dimensOk ? width / height : 16 / 9,
      getDimensions: !dimensOk && _widget.webViewJs == true,
      interceptNavigationRequest: (newUrl) {
        if (newUrl == url) return false;

        buildGestureTapCallbackForUrl(newUrl)();
        return true;
      },
      js: _widget.webViewJs == true,
      unsupportedWorkaroundForIssue37:
          _widget.unsupportedWebViewWorkaroundForIssue37 == true,
    );
  }

  Widget buildWebViewLinkOnly(String url) => GestureDetector(
        child: Text(url),
        onTap: buildGestureTapCallbackForUrl(url),
      );

  @override
  void parseTag(
    NodeMetadata meta,
    String tag,
    Map<dynamic, String> attributes,
  ) {
    switch (tag) {
      case 'a':
        meta.op = tagAExtended();
        break;
      case 'iframe':
        meta.op = tagIframe();
        // return asap to avoid being disabled by core
        return;
      case 'svg':
        meta.op = tagSvg();
        // return asap to avoid being disabled by core
        return;
      case 'video':
        meta.op = tagVideo();
        break;
    }

    return super.parseTag(meta, tag, attributes);
  }

  @override
  void reset(core.HtmlWidget widget) {
    if (widget is HtmlWidget) _widget = widget;
    super.reset(widget);
  }

  BuildOp tagAExtended() {
    _tagAExtended ??= _TagAExtended().buildOp;
    return _tagAExtended;
  }

  BuildOp tagIframe() {
    _tagIframe ??= _TagIframe(this).buildOp;
    return _tagIframe;
  }

  BuildOp tagSvg() {
    _tagSvg ??= _TagSvg(this).buildOp;
    return _tagSvg;
  }

  BuildOp tagVideo() {
    _tagVideo ??= _TagVideo(this).buildOp;
    return _tagVideo;
  }
}
